from .snappicommon import SnappiObject
from .laglacp import LagLacp
from .lagstatic import LagStatic


class LagProtocol(SnappiObject):
    _TYPES = {
        'lacp': '.laglacp.LagLacp',
        'static': '.lagstatic.LagStatic',
    }

    LACP = 'lacp'
    STATIC = 'static'

    def __init__(self):
        super(LagProtocol, self).__init__()

    @property
    def lacp(self):
        # type: () -> LagLacp
        """Factory method to create an instance of the LagLacp class

        TBD
        """
        if 'lacp' not in self._properties or self._properties['lacp'] is None:
            self._properties['lacp'] = LagLacp()
        self.choice = 'lacp'
        return self._properties['lacp']

    @property
    def static(self):
        # type: () -> LagStatic
        """Factory method to create an instance of the LagStatic class

        TBD
        """
        if 'static' not in self._properties or self._properties['static'] is None:
            self._properties['static'] = LagStatic()
        self.choice = 'static'
        return self._properties['static']

    @property
    def choice(self):
        # type: () -> Union[lacp, static, choice, choice, choice]
        """choice getter

        The type of lag protocol.

        Returns: Union[lacp, static, choice, choice, choice]
        """
        return self._properties['choice']

    @choice.setter
    def choice(self, value):
        """choice setter

        The type of lag protocol.

        value: Union[lacp, static, choice, choice, choice]
        """
        self._properties['choice'] = value



from .snappicommon import SnappiObject
from .flowport import FlowPort
from .flowdevice import FlowDevice


class FlowTxRx(SnappiObject):
    _TYPES = {
        'port': '.flowport.FlowPort',
        'device': '.flowdevice.FlowDevice',
    }

    PORT = 'port'
    DEVICE = 'device'

    def __init__(self):
        super(FlowTxRx, self).__init__()

    @property
    def port(self):
        # type: () -> FlowPort
        """Factory method to create an instance of the FlowPort class

        A container for a transmit port and 0..n intended receive ports. When assigning this container to a flow the flows's packet headers will not be populated with any address resolution information such as source and/or destination addresses. For example Flow.Ethernet dst mac address values will be defaulted to 0. For full control over the Flow.properties.packet header contents use this container. 
        """
        if 'port' not in self._properties or self._properties['port'] is None:
            self._properties['port'] = FlowPort()
        self.choice = 'port'
        return self._properties['port']

    @property
    def device(self):
        # type: () -> FlowDevice
        """Factory method to create an instance of the FlowDevice class

        A container for 1..n transmit devices and 1..n receive devices. Implemementations may use learned information from the devices to pre-populate Flow.properties.packet[Flow.Header fields].. For example an implementation may automatically start devices, get arp table information and pre-populate the Flow.Ethernet dst mac address values.. To discover what the implementation supports use the /results/capabilities API.
        """
        if 'device' not in self._properties or self._properties['device'] is None:
            self._properties['device'] = FlowDevice()
        self.choice = 'device'
        return self._properties['device']

    @property
    def choice(self):
        # type: () -> Union[port, device, choice, choice, choice]
        """choice getter

        The type of transmit and receive container used by the flow.

        Returns: Union[port, device, choice, choice, choice]
        """
        return self._properties['choice']

    @choice.setter
    def choice(self, value):
        """choice setter

        The type of transmit and receive container used by the flow.

        value: Union[port, device, choice, choice, choice]
        """
        self._properties['choice'] = value



from .snappicommon import SnappiObject


class FlowBitCounter(SnappiObject):
    def __init__(self, offset=None, length=None, count=None, start=None, step=None):
        super(FlowBitCounter, self).__init__()
        self.offset = offset
        self.length = length
        self.count = count
        self.start = start
        self.step = step

    @property
    def offset(self):
        # type: () -> int
        """offset getter

        Bit offset in the packet at which the pattern will be applied

        Returns: int
        """
        return self._properties['offset']

    @offset.setter
    def offset(self, value):
        """offset setter

        Bit offset in the packet at which the pattern will be applied

        value: int
        """
        self._properties['offset'] = value

    @property
    def length(self):
        # type: () -> int
        """length getter

        The number of bits in the packet that the pattern will span

        Returns: int
        """
        return self._properties['length']

    @length.setter
    def length(self, value):
        """length setter

        The number of bits in the packet that the pattern will span

        value: int
        """
        self._properties['length'] = value

    @property
    def count(self):
        # type: () -> int
        """count getter

        The number of values to generate before repeating A value of 0 means the pattern will count continuously

        Returns: int
        """
        return self._properties['count']

    @count.setter
    def count(self, value):
        """count setter

        The number of values to generate before repeating A value of 0 means the pattern will count continuously

        value: int
        """
        self._properties['count'] = value

    @property
    def start(self):
        # type: () -> str
        """start getter

        The starting value of the pattern. If the value is greater than the length it will be truncated.

        Returns: str
        """
        return self._properties['start']

    @start.setter
    def start(self, value):
        """start setter

        The starting value of the pattern. If the value is greater than the length it will be truncated.

        value: str
        """
        self._properties['start'] = value

    @property
    def step(self):
        # type: () -> str
        """step getter

        The amount the start value will be incremented by If the value is greater than the length it will be truncated.

        Returns: str
        """
        return self._properties['step']

    @step.setter
    def step(self, value):
        """step setter

        The amount the start value will be incremented by If the value is greater than the length it will be truncated.

        value: str
        """
        self._properties['step'] = value



from .snappicommon import SnappiObject


class FlowDevice(SnappiObject):
    def __init__(self, tx_names=None, rx_names=None):
        super(FlowDevice, self).__init__()
        self.tx_names = tx_names
        self.rx_names = rx_names

    @property
    def tx_names(self):
        # type: () -> list[str]
        """tx_names getter

        The unique names of devices that will be transmitting.

        Returns: list[str]
        """
        return self._properties['tx_names']

    @tx_names.setter
    def tx_names(self, value):
        """tx_names setter

        The unique names of devices that will be transmitting.

        value: list[str]
        """
        self._properties['tx_names'] = value

    @property
    def rx_names(self):
        # type: () -> list[str]
        """rx_names getter

        The unique names of emulated devices that will be receiving.

        Returns: list[str]
        """
        return self._properties['rx_names']

    @rx_names.setter
    def rx_names(self, value):
        """rx_names setter

        The unique names of emulated devices that will be receiving.

        value: list[str]
        """
        self._properties['rx_names'] = value



from .snappicommon import SnappiObject
from .portoptions import PortOptions


class ConfigOptions(SnappiObject):
    _TYPES = {
        'port_options': '.portoptions.PortOptions',
    }

    def __init__(self):
        super(ConfigOptions, self).__init__()

    @property
    def port_options(self):
        # type: () -> PortOptions
        """port_options getter

        Common port options that apply to all configured Port objects. 

        Returns: obj(snappi.PortOptions)
        """
        if 'port_options' not in self._properties or self._properties['port_options'] is None:
            self._properties['port_options'] = PortOptions()
        return self._properties['port_options']



from .flowpattern import FlowPattern
from .snappicommon import SnappiObject


class FlowUdp(SnappiObject):
    _TYPES = {
        'src_port': '.flowpattern.FlowPattern',
        'dst_port': '.flowpattern.FlowPattern',
        'length': '.flowpattern.FlowPattern',
        'checksum': '.flowpattern.FlowPattern',
    }

    def __init__(self):
        super(FlowUdp, self).__init__()

    @property
    def src_port(self):
        # type: () -> FlowPattern
        """src_port getter

        A container for packet header field patterns.A container for packet header field patterns.Udp source port. Max length is 2 bytes.

        Returns: obj(snappi.FlowPattern)
        """
        if 'src_port' not in self._properties or self._properties['src_port'] is None:
            self._properties['src_port'] = FlowPattern()
        return self._properties['src_port']

    @property
    def dst_port(self):
        # type: () -> FlowPattern
        """dst_port getter

        A container for packet header field patterns.A container for packet header field patterns.Tcp destination port. Max length is 2 bytes.

        Returns: obj(snappi.FlowPattern)
        """
        if 'dst_port' not in self._properties or self._properties['dst_port'] is None:
            self._properties['dst_port'] = FlowPattern()
        return self._properties['dst_port']

    @property
    def length(self):
        # type: () -> FlowPattern
        """length getter

        A container for packet header field patterns.A container for packet header field patterns.Length in bytes of the udp header and yudp data. Max length is 2 bytes.

        Returns: obj(snappi.FlowPattern)
        """
        if 'length' not in self._properties or self._properties['length'] is None:
            self._properties['length'] = FlowPattern()
        return self._properties['length']

    @property
    def checksum(self):
        # type: () -> FlowPattern
        """checksum getter

        A container for packet header field patterns.A container for packet header field patterns.Checksum field used for error checking of header and data. Max length is 2 bytes.

        Returns: obj(snappi.FlowPattern)
        """
        if 'checksum' not in self._properties or self._properties['checksum'] is None:
            self._properties['checksum'] = FlowPattern()
        return self._properties['checksum']



from .snappicommon import SnappiObject


class CaptureMacAddressFilter(SnappiObject):
    SOURCE = 'source'
    DESTINATION = 'destination'

    def __init__(self, mac=None, filter=None, mask=None):
        super(CaptureMacAddressFilter, self).__init__()
        self.mac = mac
        self.filter = filter
        self.mask = mask

    @property
    def mac(self):
        # type: () -> Union[source, destination]
        """mac getter

        The type of mac address filters. This can be either source or destination.

        Returns: Union[source, destination]
        """
        return self._properties['mac']

    @mac.setter
    def mac(self, value):
        """mac setter

        The type of mac address filters. This can be either source or destination.

        value: Union[source, destination]
        """
        self._properties['mac'] = value

    @property
    def filter(self):
        # type: () -> str
        """filter getter

        The value of the mac address.

        Returns: str
        """
        return self._properties['filter']

    @filter.setter
    def filter(self, value):
        """filter setter

        The value of the mac address.

        value: str
        """
        self._properties['filter'] = value

    @property
    def mask(self):
        # type: () -> str
        """mask getter

        The value of the mask to be applied to the mac address.

        Returns: str
        """
        return self._properties['mask']

    @mask.setter
    def mask(self, value):
        """mask setter

        The value of the mask to be applied to the mac address.

        value: str
        """
        self._properties['mask'] = value



from .snappicommon import SnappiObject


class FlowMetricsRequest(SnappiObject):
    TRANSMIT = 'transmit'
    PORT_TX = 'port_tx'
    PORT_RX = 'port_rx'
    FRAMES_TX = 'frames_tx'
    FRAMES_RX = 'frames_rx'
    BYTES_TX = 'bytes_tx'
    BYTES_RX = 'bytes_rx'
    FRAMES_TX_RATE = 'frames_tx_rate'
    FRAMES_RX_RATE = 'frames_rx_rate'
    LOSS = 'loss'

    def __init__(self, flow_names=None, column_names=None, result_group_names=None):
        super(FlowMetricsRequest, self).__init__()
        self.flow_names = flow_names
        self.column_names = column_names
        self.result_group_names = result_group_names

    @property
    def flow_names(self):
        # type: () -> list[str]
        """flow_names getter

        The names of flow objects to return results for. An empty list will return results for all flows.

        Returns: list[str]
        """
        return self._properties['flow_names']

    @flow_names.setter
    def flow_names(self, value):
        """flow_names setter

        The names of flow objects to return results for. An empty list will return results for all flows.

        value: list[str]
        """
        self._properties['flow_names'] = value

    @property
    def column_names(self):
        # type: () -> list[Union[transmit, port_tx, port_rx, frames_tx, frames_rx, bytes_tx, bytes_rx, frames_tx_rate, frames_rx_rate, loss]]
        """column_names getter

        The list of column names that the returned result set will contain. If the list is empty then all columns will be returned except for any result_groups. The name of the flow cannot be excluded.

        Returns: list[Union[transmit, port_tx, port_rx, frames_tx, frames_rx, bytes_tx, bytes_rx, frames_tx_rate, frames_rx_rate, loss]]
        """
        return self._properties['column_names']

    @column_names.setter
    def column_names(self, value):
        """column_names setter

        The list of column names that the returned result set will contain. If the list is empty then all columns will be returned except for any result_groups. The name of the flow cannot be excluded.

        value: list[Union[transmit, port_tx, port_rx, frames_tx, frames_rx, bytes_tx, bytes_rx, frames_tx_rate, frames_rx_rate, loss]]
        """
        self._properties['column_names'] = value

    @property
    def result_group_names(self):
        # type: () -> list[str]
        """result_group_names getter

        Extend the details of flow results by specifying any configured flow packet header field result_group names.

        Returns: list[str]
        """
        return self._properties['result_group_names']

    @result_group_names.setter
    def result_group_names(self, value):
        """result_group_names setter

        Extend the details of flow results by specifying any configured flow packet header field result_group names.

        value: list[str]
        """
        self._properties['result_group_names'] = value



from .devicepattern import DevicePattern
from .snappicommon import SnappiObject


class LagLacp(SnappiObject):
    _TYPES = {
        'actor_key': '.devicepattern.DevicePattern',
        'actor_port_number': '.devicepattern.DevicePattern',
        'actor_port_priority': '.devicepattern.DevicePattern',
        'actor_system_id': '.devicepattern.DevicePattern',
        'actor_system_priority': '.devicepattern.DevicePattern',
    }

    def __init__(self):
        super(LagLacp, self).__init__()

    @property
    def actor_key(self):
        # type: () -> DevicePattern
        """actor_key getter

        A container for emulated device property patterns.A container for emulated device property patterns.The actor key.

        Returns: obj(snappi.DevicePattern)
        """
        if 'actor_key' not in self._properties or self._properties['actor_key'] is None:
            self._properties['actor_key'] = DevicePattern()
        return self._properties['actor_key']

    @property
    def actor_port_number(self):
        # type: () -> DevicePattern
        """actor_port_number getter

        A container for emulated device property patterns.A container for emulated device property patterns.The actor port number.

        Returns: obj(snappi.DevicePattern)
        """
        if 'actor_port_number' not in self._properties or self._properties['actor_port_number'] is None:
            self._properties['actor_port_number'] = DevicePattern()
        return self._properties['actor_port_number']

    @property
    def actor_port_priority(self):
        # type: () -> DevicePattern
        """actor_port_priority getter

        A container for emulated device property patterns.A container for emulated device property patterns.The actor port priority.

        Returns: obj(snappi.DevicePattern)
        """
        if 'actor_port_priority' not in self._properties or self._properties['actor_port_priority'] is None:
            self._properties['actor_port_priority'] = DevicePattern()
        return self._properties['actor_port_priority']

    @property
    def actor_system_id(self):
        # type: () -> DevicePattern
        """actor_system_id getter

        A container for emulated device property patterns.A container for emulated device property patterns.The actor system id.

        Returns: obj(snappi.DevicePattern)
        """
        if 'actor_system_id' not in self._properties or self._properties['actor_system_id'] is None:
            self._properties['actor_system_id'] = DevicePattern()
        return self._properties['actor_system_id']

    @property
    def actor_system_priority(self):
        # type: () -> DevicePattern
        """actor_system_priority getter

        A container for emulated device property patterns.A container for emulated device property patterns.The actor system priority.

        Returns: obj(snappi.DevicePattern)
        """
        if 'actor_system_priority' not in self._properties or self._properties['actor_system_priority'] is None:
            self._properties['actor_system_priority'] = DevicePattern()
        return self._properties['actor_system_priority']



from .snappicommon import SnappiObject
from .devicecounter import DeviceCounter


class DevicePattern(SnappiObject):
    _TYPES = {
        'increment': '.devicecounter.DeviceCounter',
        'decrement': '.devicecounter.DeviceCounter',
    }

    VALUE = 'value'
    VALUE_LIST = 'value_list'
    INCREMENT = 'increment'
    DECREMENT = 'decrement'

    def __init__(self):
        super(DevicePattern, self).__init__()

    @property
    def increment(self):
        # type: () -> DeviceCounter
        """Factory method to create an instance of the DeviceCounter class

        An incrementing pattern.
        """
        if 'increment' not in self._properties or self._properties['increment'] is None:
            self._properties['increment'] = DeviceCounter()
        self.choice = 'increment'
        return self._properties['increment']

    @property
    def decrement(self):
        # type: () -> DeviceCounter
        """Factory method to create an instance of the DeviceCounter class

        An incrementing pattern.
        """
        if 'decrement' not in self._properties or self._properties['decrement'] is None:
            self._properties['decrement'] = DeviceCounter()
        self.choice = 'decrement'
        return self._properties['decrement']

    @property
    def choice(self):
        # type: () -> Union[value, value_list, increment, decrement, choice, choice, choice]
        """choice getter

        TBD

        Returns: Union[value, value_list, increment, decrement, choice, choice, choice]
        """
        return self._properties['choice']

    @choice.setter
    def choice(self, value):
        """choice setter

        TBD

        value: Union[value, value_list, increment, decrement, choice, choice, choice]
        """
        self._properties['choice'] = value

    @property
    def value(self):
        # type: () -> str
        """value getter

        TBD

        Returns: str
        """
        return self._properties['value']

    @value.setter
    def value(self, value):
        """value setter

        TBD

        value: str
        """
        self._properties['choice'] = 'value'
        self._properties['value'] = value

    @property
    def value_list(self):
        # type: () -> list[str]
        """value_list getter

        TBD

        Returns: list[str]
        """
        return self._properties['value_list']

    @value_list.setter
    def value_list(self, value):
        """value_list setter

        TBD

        value: list[str]
        """
        self._properties['choice'] = 'value_list'
        self._properties['value_list'] = value



from .snappicommon import SnappiList
from .flowgtpextension import FlowGtpExtension


class FlowGtpExtensionList(SnappiList):
    def __init__(self):
        super(FlowGtpExtensionList, self).__init__()


    def gtpextension(self):
        # type: () -> FlowGtpExtension
        """Factory method to create an instance of the snappi.flowgtpextension.FlowGtpExtension class

        TBD
        """
        item = FlowGtpExtension()
        self._add(item)
        return self



from .flowstatelist import FlowStateList
from .snappicommon import SnappiObject
from .portstatelist import PortStateList


class StateMetrics(SnappiObject):
    _TYPES = {
        'port_state': '.portstatelist.PortStateList',
        'flow_state': '.flowstatelist.FlowStateList',
    }

    def __init__(self):
        super(StateMetrics, self).__init__()

    @property
    def port_state(self):
        # type: () -> PortStateList
        """port_state getter

        TBD

        Returns: list[obj(snappi.PortState)]
        """
        if 'port_state' not in self._properties or self._properties['port_state'] is None:
            self._properties['port_state'] = PortStateList()
        return self._properties['port_state']

    @property
    def flow_state(self):
        # type: () -> FlowStateList
        """flow_state getter

        TBD

        Returns: list[obj(snappi.FlowState)]
        """
        if 'flow_state' not in self._properties or self._properties['flow_state'] is None:
            self._properties['flow_state'] = FlowStateList()
        return self._properties['flow_state']



from .snappicommon import SnappiObject
from .bgpv4metriclist import Bgpv4MetricList


class Bgpv4Metrics(SnappiObject):
    _TYPES = {
        'ports': '.bgpv4metriclist.Bgpv4MetricList',
    }

    def __init__(self):
        super(Bgpv4Metrics, self).__init__()

    @property
    def ports(self):
        # type: () -> Bgpv4MetricList
        """ports getter

        TBD

        Returns: list[obj(snappi.Bgpv4Metric)]
        """
        if 'ports' not in self._properties or self._properties['ports'] is None:
            self._properties['ports'] = Bgpv4MetricList()
        return self._properties['ports']



from .snappicommon import SnappiList
from .flowstate import FlowState


class FlowStateList(SnappiList):
    def __init__(self):
        super(FlowStateList, self).__init__()


    def state(self, name=None, transmit='None'):
        # type: () -> FlowState
        """Factory method to create an instance of the snappi.flowstate.FlowState class

        TBD
        """
        item = FlowState(name, transmit)
        self._add(item)
        return self



from .lag import Lag
from .snappicommon import SnappiList


class LagList(SnappiList):
    def __init__(self):
        super(LagList, self).__init__()


    def lag(self, port_names=None, name=None):
        # type: () -> Lag
        """Factory method to create an instance of the snappi.lag.Lag class

        A container for LAG settings.
        """
        item = Lag(port_names, name)
        self._add(item)
        return self



from .flowethernetpause import FlowEthernetPause
from .snappicommon import SnappiList
from .flowcustom import FlowCustom
from .flowgtpv2 import FlowGtpv2
from .flowtcp import FlowTcp
from .flowudp import FlowUdp
from .flowheader import FlowHeader
from .flowipv6 import FlowIpv6
from .flowgre import FlowGre
from .flowethernet import FlowEthernet
from .flowgtpv1 import FlowGtpv1
from .flowvxlan import FlowVxlan
from .flowvlan import FlowVlan
from .flowpfcpause import FlowPfcPause
from .flowipv4 import FlowIpv4


class FlowHeaderList(SnappiList):
    def __init__(self):
        super(FlowHeaderList, self).__init__()


    def header(self):
        # type: () -> FlowHeader
        """Factory method to create an instance of the snappi.flowheader.FlowHeader class

        Container for all traffic packet headers
        """
        item = FlowHeader()
        self._add(item)
        return self

    def custom(self, bytes=None):
        # type: () -> FlowHeaderList
        """Factory method to create an instance of the snappi.flowcustom.FlowCustom class

        Custom packet header
        """
        item = FlowHeader()
        item.custom
        self._add(item)
        return self

    def ethernet(self):
        # type: () -> FlowHeaderList
        """Factory method to create an instance of the snappi.flowethernet.FlowEthernet class

        Ethernet packet header
        """
        item = FlowHeader()
        item.ethernet
        self._add(item)
        return self

    def vlan(self):
        # type: () -> FlowHeaderList
        """Factory method to create an instance of the snappi.flowvlan.FlowVlan class

        Vlan packet header
        """
        item = FlowHeader()
        item.vlan
        self._add(item)
        return self

    def vxlan(self):
        # type: () -> FlowHeaderList
        """Factory method to create an instance of the snappi.flowvxlan.FlowVxlan class

        Vxlan packet header
        """
        item = FlowHeader()
        item.vxlan
        self._add(item)
        return self

    def ipv4(self):
        # type: () -> FlowHeaderList
        """Factory method to create an instance of the snappi.flowipv4.FlowIpv4 class

        Ipv4 packet header
        """
        item = FlowHeader()
        item.ipv4
        self._add(item)
        return self

    def ipv6(self):
        # type: () -> FlowHeaderList
        """Factory method to create an instance of the snappi.flowipv6.FlowIpv6 class

        Ipv6 packet header
        """
        item = FlowHeader()
        item.ipv6
        self._add(item)
        return self

    def pfcpause(self):
        # type: () -> FlowHeaderList
        """Factory method to create an instance of the snappi.flowpfcpause.FlowPfcPause class

        IEEE 802.1Qbb PFC Pause packet header. - dst: 01:80:C2:00:00:01 48bits - src: 48bits - ether_type: 0x8808 16bits - control_op_code: 0x0101 16bits - class_enable_vector: 16bits - pause_class_0: 0x0000 16bits - pause_class_1: 0x0000 16bits - pause_class_2: 0x0000 16bits - pause_class_3: 0x0000 16bits - pause_class_4: 0x0000 16bits - pause_class_5: 0x0000 16bits - pause_class_6: 0x0000 16bits - pause_class_7: 0x0000 16bits
        """
        item = FlowHeader()
        item.pfcpause
        self._add(item)
        return self

    def ethernetpause(self):
        # type: () -> FlowHeaderList
        """Factory method to create an instance of the snappi.flowethernetpause.FlowEthernetPause class

        IEEE 802.3x Ethernet Pause packet header. - dst: 01:80:C2:00:00:01 48bits - src: 48bits - ether_type: 0x8808 16bits - control_op_code: 0x0001 16bits - time: 0x0000 16bits
        """
        item = FlowHeader()
        item.ethernetpause
        self._add(item)
        return self

    def tcp(self):
        # type: () -> FlowHeaderList
        """Factory method to create an instance of the snappi.flowtcp.FlowTcp class

        Tcp packet header
        """
        item = FlowHeader()
        item.tcp
        self._add(item)
        return self

    def udp(self):
        # type: () -> FlowHeaderList
        """Factory method to create an instance of the snappi.flowudp.FlowUdp class

        Udp packet header
        """
        item = FlowHeader()
        item.udp
        self._add(item)
        return self

    def gre(self):
        # type: () -> FlowHeaderList
        """Factory method to create an instance of the snappi.flowgre.FlowGre class

        Gre packet header
        """
        item = FlowHeader()
        item.gre
        self._add(item)
        return self

    def gtpv1(self):
        # type: () -> FlowHeaderList
        """Factory method to create an instance of the snappi.flowgtpv1.FlowGtpv1 class

        GTPv1 packet header
        """
        item = FlowHeader()
        item.gtpv1
        self._add(item)
        return self

    def gtpv2(self):
        # type: () -> FlowHeaderList
        """Factory method to create an instance of the snappi.flowgtpv2.FlowGtpv2 class

        GTPv2 packet header
        """
        item = FlowHeader()
        item.gtpv2
        self._add(item)
        return self



from .snappicommon import SnappiObject


class FlowBurst(SnappiObject):
    BYTES = 'bytes'
    NANOSECONDS = 'nanoseconds'

    def __init__(self, packets=None, gap=None, inter_burst_gap=None, inter_burst_gap_unit=None):
        super(FlowBurst, self).__init__()
        self.packets = packets
        self.gap = gap
        self.inter_burst_gap = inter_burst_gap
        self.inter_burst_gap_unit = inter_burst_gap_unit

    @property
    def packets(self):
        # type: () -> int
        """packets getter

        The number of packets transmitted per burst.

        Returns: int
        """
        return self._properties['packets']

    @packets.setter
    def packets(self, value):
        """packets setter

        The number of packets transmitted per burst.

        value: int
        """
        self._properties['packets'] = value

    @property
    def gap(self):
        # type: () -> int
        """gap getter

        The minimum gap between packets expressed as bytes.

        Returns: int
        """
        return self._properties['gap']

    @gap.setter
    def gap(self, value):
        """gap setter

        The minimum gap between packets expressed as bytes.

        value: int
        """
        self._properties['gap'] = value

    @property
    def inter_burst_gap(self):
        # type: () -> int
        """inter_burst_gap getter

        The gap between the transmission of each burst. A value of 0 means there is no gap between bursts.

        Returns: int
        """
        return self._properties['inter_burst_gap']

    @inter_burst_gap.setter
    def inter_burst_gap(self, value):
        """inter_burst_gap setter

        The gap between the transmission of each burst. A value of 0 means there is no gap between bursts.

        value: int
        """
        self._properties['inter_burst_gap'] = value

    @property
    def inter_burst_gap_unit(self):
        # type: () -> Union[bytes, nanoseconds]
        """inter_burst_gap_unit getter

        The inter burst gap expressed as a number of this value.

        Returns: Union[bytes, nanoseconds]
        """
        return self._properties['inter_burst_gap_unit']

    @inter_burst_gap_unit.setter
    def inter_burst_gap_unit(self, value):
        """inter_burst_gap_unit setter

        The inter burst gap expressed as a number of this value.

        value: Union[bytes, nanoseconds]
        """
        self._properties['inter_burst_gap_unit'] = value



from .flowethernetpause import FlowEthernetPause
from .flowcustom import FlowCustom
from .flowgtpv2 import FlowGtpv2
from .snappicommon import SnappiObject
from .flowtcp import FlowTcp
from .flowudp import FlowUdp
from .flowgre import FlowGre
from .flowipv6 import FlowIpv6
from .flowethernet import FlowEthernet
from .flowgtpv1 import FlowGtpv1
from .flowvxlan import FlowVxlan
from .flowvlan import FlowVlan
from .flowpfcpause import FlowPfcPause
from .flowipv4 import FlowIpv4


class FlowHeader(SnappiObject):
    _TYPES = {
        'custom': '.flowcustom.FlowCustom',
        'ethernet': '.flowethernet.FlowEthernet',
        'vlan': '.flowvlan.FlowVlan',
        'vxlan': '.flowvxlan.FlowVxlan',
        'ipv4': '.flowipv4.FlowIpv4',
        'ipv6': '.flowipv6.FlowIpv6',
        'pfcpause': '.flowpfcpause.FlowPfcPause',
        'ethernetpause': '.flowethernetpause.FlowEthernetPause',
        'tcp': '.flowtcp.FlowTcp',
        'udp': '.flowudp.FlowUdp',
        'gre': '.flowgre.FlowGre',
        'gtpv1': '.flowgtpv1.FlowGtpv1',
        'gtpv2': '.flowgtpv2.FlowGtpv2',
    }

    CUSTOM = 'custom'
    ETHERNET = 'ethernet'
    VLAN = 'vlan'
    VXLAN = 'vxlan'
    IPV4 = 'ipv4'
    IPV6 = 'ipv6'
    PFCPAUSE = 'pfcpause'
    ETHERNETPAUSE = 'ethernetpause'
    TCP = 'tcp'
    UDP = 'udp'
    GRE = 'gre'
    GTPV1 = 'gtpv1'
    GTPV2 = 'gtpv2'

    def __init__(self):
        super(FlowHeader, self).__init__()

    @property
    def custom(self):
        # type: () -> FlowCustom
        """Factory method to create an instance of the FlowCustom class

        Custom packet header
        """
        if 'custom' not in self._properties or self._properties['custom'] is None:
            self._properties['custom'] = FlowCustom()
        self.choice = 'custom'
        return self._properties['custom']

    @property
    def ethernet(self):
        # type: () -> FlowEthernet
        """Factory method to create an instance of the FlowEthernet class

        Ethernet packet header
        """
        if 'ethernet' not in self._properties or self._properties['ethernet'] is None:
            self._properties['ethernet'] = FlowEthernet()
        self.choice = 'ethernet'
        return self._properties['ethernet']

    @property
    def vlan(self):
        # type: () -> FlowVlan
        """Factory method to create an instance of the FlowVlan class

        Vlan packet header
        """
        if 'vlan' not in self._properties or self._properties['vlan'] is None:
            self._properties['vlan'] = FlowVlan()
        self.choice = 'vlan'
        return self._properties['vlan']

    @property
    def vxlan(self):
        # type: () -> FlowVxlan
        """Factory method to create an instance of the FlowVxlan class

        Vxlan packet header
        """
        if 'vxlan' not in self._properties or self._properties['vxlan'] is None:
            self._properties['vxlan'] = FlowVxlan()
        self.choice = 'vxlan'
        return self._properties['vxlan']

    @property
    def ipv4(self):
        # type: () -> FlowIpv4
        """Factory method to create an instance of the FlowIpv4 class

        Ipv4 packet header
        """
        if 'ipv4' not in self._properties or self._properties['ipv4'] is None:
            self._properties['ipv4'] = FlowIpv4()
        self.choice = 'ipv4'
        return self._properties['ipv4']

    @property
    def ipv6(self):
        # type: () -> FlowIpv6
        """Factory method to create an instance of the FlowIpv6 class

        Ipv6 packet header
        """
        if 'ipv6' not in self._properties or self._properties['ipv6'] is None:
            self._properties['ipv6'] = FlowIpv6()
        self.choice = 'ipv6'
        return self._properties['ipv6']

    @property
    def pfcpause(self):
        # type: () -> FlowPfcPause
        """Factory method to create an instance of the FlowPfcPause class

        IEEE 802.1Qbb PFC Pause packet header. - dst: 01:80:C2:00:00:01 48bits - src: 48bits - ether_type: 0x8808 16bits - control_op_code: 0x0101 16bits - class_enable_vector: 16bits - pause_class_0: 0x0000 16bits - pause_class_1: 0x0000 16bits - pause_class_2: 0x0000 16bits - pause_class_3: 0x0000 16bits - pause_class_4: 0x0000 16bits - pause_class_5: 0x0000 16bits - pause_class_6: 0x0000 16bits - pause_class_7: 0x0000 16bits
        """
        if 'pfcpause' not in self._properties or self._properties['pfcpause'] is None:
            self._properties['pfcpause'] = FlowPfcPause()
        self.choice = 'pfcpause'
        return self._properties['pfcpause']

    @property
    def ethernetpause(self):
        # type: () -> FlowEthernetPause
        """Factory method to create an instance of the FlowEthernetPause class

        IEEE 802.3x Ethernet Pause packet header. - dst: 01:80:C2:00:00:01 48bits - src: 48bits - ether_type: 0x8808 16bits - control_op_code: 0x0001 16bits - time: 0x0000 16bits
        """
        if 'ethernetpause' not in self._properties or self._properties['ethernetpause'] is None:
            self._properties['ethernetpause'] = FlowEthernetPause()
        self.choice = 'ethernetpause'
        return self._properties['ethernetpause']

    @property
    def tcp(self):
        # type: () -> FlowTcp
        """Factory method to create an instance of the FlowTcp class

        Tcp packet header
        """
        if 'tcp' not in self._properties or self._properties['tcp'] is None:
            self._properties['tcp'] = FlowTcp()
        self.choice = 'tcp'
        return self._properties['tcp']

    @property
    def udp(self):
        # type: () -> FlowUdp
        """Factory method to create an instance of the FlowUdp class

        Udp packet header
        """
        if 'udp' not in self._properties or self._properties['udp'] is None:
            self._properties['udp'] = FlowUdp()
        self.choice = 'udp'
        return self._properties['udp']

    @property
    def gre(self):
        # type: () -> FlowGre
        """Factory method to create an instance of the FlowGre class

        Gre packet header
        """
        if 'gre' not in self._properties or self._properties['gre'] is None:
            self._properties['gre'] = FlowGre()
        self.choice = 'gre'
        return self._properties['gre']

    @property
    def gtpv1(self):
        # type: () -> FlowGtpv1
        """Factory method to create an instance of the FlowGtpv1 class

        GTPv1 packet header
        """
        if 'gtpv1' not in self._properties or self._properties['gtpv1'] is None:
            self._properties['gtpv1'] = FlowGtpv1()
        self.choice = 'gtpv1'
        return self._properties['gtpv1']

    @property
    def gtpv2(self):
        # type: () -> FlowGtpv2
        """Factory method to create an instance of the FlowGtpv2 class

        GTPv2 packet header
        """
        if 'gtpv2' not in self._properties or self._properties['gtpv2'] is None:
            self._properties['gtpv2'] = FlowGtpv2()
        self.choice = 'gtpv2'
        return self._properties['gtpv2']

    @property
    def choice(self):
        # type: () -> Union[custom, ethernet, vlan, vxlan, ipv4, ipv6, pfcpause, ethernetpause, tcp, udp, gre, gtpv1, gtpv2, choice, choice, choice]
        """choice getter

        TBD

        Returns: Union[custom, ethernet, vlan, vxlan, ipv4, ipv6, pfcpause, ethernetpause, tcp, udp, gre, gtpv1, gtpv2, choice, choice, choice]
        """
        return self._properties['choice']

    @choice.setter
    def choice(self, value):
        """choice setter

        TBD

        value: Union[custom, ethernet, vlan, vxlan, ipv4, ipv6, pfcpause, ethernetpause, tcp, udp, gre, gtpv1, gtpv2, choice, choice, choice]
        """
        self._properties['choice'] = value



"""Build distributions

To build `python setup.py sdist --formats=gztar bdist_wheel --universal`
"""
import os
from setuptools import setup, find_packages
from snappi.snappigenerator import SnappiGenerator

pkg_name = 'snappi'

SnappiGenerator()

base_dir = os.path.dirname(os.path.abspath(__file__))
with open(os.path.join(base_dir, 'README.md')) as fid:
    long_description = fid.read()
with open(os.path.join(base_dir, 'VERSION')) as fid:
    version_number = fid.read()

setup(name=pkg_name,
      version=version_number,
      description='The Snappi Open Traffic Generator Python Package',
      long_description=long_description,
      long_description_content_type='text/markdown',
      url='https://github.com/open-traffic-generator/snappi',
      author='ajbalogh',
      author_email='andy.balogh@keysight.com',
      license='MIT',
      classifiers=[
          'Development Status :: 3 - Alpha', 'Intended Audience :: Developers',
          'Topic :: Software Development :: Testing :: Traffic Generation',
          'License :: OSI Approved :: MIT License',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3'
      ],
      keywords='snappi testing open traffic generator automation',
      include_package_data=True,
      packages=find_packages(),
      python_requires='>=2.7, <4',
      install_requires=['requests', 'pyyaml', 'jsonpath-ng'],
      tests_require=['pytest'])



from .devicepattern import DevicePattern
from .snappicommon import SnappiObject


class DeviceBgpv6RouteRange(SnappiObject):
    _TYPES = {
        'address': '.devicepattern.DevicePattern',
        'prefix': '.devicepattern.DevicePattern',
        'as_path': '.devicepattern.DevicePattern',
        'next_hop_address': '.devicepattern.DevicePattern',
        'community': '.devicepattern.DevicePattern',
    }

    def __init__(self, route_count_per_device=None, name=None):
        super(DeviceBgpv6RouteRange, self).__init__()
        self.route_count_per_device = route_count_per_device
        self.name = name

    @property
    def route_count_per_device(self):
        # type: () -> int
        """route_count_per_device getter

        The number of routes per device.

        Returns: int
        """
        return self._properties['route_count_per_device']

    @route_count_per_device.setter
    def route_count_per_device(self, value):
        """route_count_per_device setter

        The number of routes per device.

        value: int
        """
        self._properties['route_count_per_device'] = value

    @property
    def address(self):
        # type: () -> DevicePattern
        """address getter

        A container for emulated device property patterns.The network address of the first network

        Returns: obj(snappi.DevicePattern)
        """
        if 'address' not in self._properties or self._properties['address'] is None:
            self._properties['address'] = DevicePattern()
        return self._properties['address']

    @property
    def prefix(self):
        # type: () -> DevicePattern
        """prefix getter

        A container for emulated device property patterns.Ipv6 prefix length with minimum value is 0 to maximum value is 128

        Returns: obj(snappi.DevicePattern)
        """
        if 'prefix' not in self._properties or self._properties['prefix'] is None:
            self._properties['prefix'] = DevicePattern()
        return self._properties['prefix']

    @property
    def as_path(self):
        # type: () -> DevicePattern
        """as_path getter

        A container for emulated device property patterns.Autonomous Systems (AS) numbers that a route passes through to reach the destination

        Returns: obj(snappi.DevicePattern)
        """
        if 'as_path' not in self._properties or self._properties['as_path'] is None:
            self._properties['as_path'] = DevicePattern()
        return self._properties['as_path']

    @property
    def next_hop_address(self):
        # type: () -> DevicePattern
        """next_hop_address getter

        A container for emulated device property patterns.IP Address of next router to forward a packet to its final destination

        Returns: obj(snappi.DevicePattern)
        """
        if 'next_hop_address' not in self._properties or self._properties['next_hop_address'] is None:
            self._properties['next_hop_address'] = DevicePattern()
        return self._properties['next_hop_address']

    @property
    def community(self):
        # type: () -> DevicePattern
        """community getter

        A container for emulated device property patterns.BGP communities provide additional capability for tagging routes and for modifying BGP routing policy on upstream and downstream routers BGP community is a 32-bit number which broken into 16-bit As and 16-bit custom value Please specify those two values in this string format 65000:100

        Returns: obj(snappi.DevicePattern)
        """
        if 'community' not in self._properties or self._properties['community'] is None:
            self._properties['community'] = DevicePattern()
        return self._properties['community']

    @property
    def name(self):
        # type: () -> str
        """name getter

        Globally unique name of an object. It also serves as the primary key for arrays of objects.

        Returns: str
        """
        return self._properties['name']

    @name.setter
    def name(self, value):
        """name setter

        Globally unique name of an object. It also serves as the primary key for arrays of objects.

        value: str
        """
        self._properties['name'] = value



from .snappicommon import SnappiList
from .devicevlan import DeviceVlan


class DeviceVlanList(SnappiList):
    def __init__(self):
        super(DeviceVlanList, self).__init__()


    def vlan(self, name=None):
        # type: () -> DeviceVlan
        """Factory method to create an instance of the snappi.devicevlan.DeviceVlan class

        Emulated vlan protocol
        """
        item = DeviceVlan(name)
        self._add(item)
        return self



from .snappicommon import SnappiObject


class DeviceCounter(SnappiObject):
    def __init__(self, start=None, step=None):
        super(DeviceCounter, self).__init__()
        self.start = start
        self.step = step

    @property
    def start(self):
        # type: () -> str
        """start getter

        TBD

        Returns: str
        """
        return self._properties['start']

    @start.setter
    def start(self, value):
        """start setter

        TBD

        value: str
        """
        self._properties['start'] = value

    @property
    def step(self):
        # type: () -> str
        """step getter

        TBD

        Returns: str
        """
        return self._properties['step']

    @step.setter
    def step(self, value):
        """step setter

        TBD

        value: str
        """
        self._properties['step'] = value



from .snappicommon import SnappiList
from .deviceethernet import DeviceEthernet
from .deviceipv6 import DeviceIpv6
from .devicecontaineripv4 import DeviceContainerIpv4
from .device import Device
from .devicebgpv4 import DeviceBgpv4


class DeviceList(SnappiList):
    def __init__(self):
        super(DeviceList, self).__init__()


    def device(self, container_name=None, device_count=1, name=None):
        # type: () -> Device
        """Factory method to create an instance of the snappi.device.Device class

        A container for emulated protocol devices.
        """
        item = Device(container_name, device_count, name)
        self._add(item)
        return self

    def ethernet(self, name=None):
        # type: () -> DeviceList
        """Factory method to create an instance of the snappi.deviceethernet.DeviceEthernet class

        Emulated ethernet protocol. A top level in the emulated device stack.
        """
        item = Device()
        item.ethernet
        self._add(item)
        return self

    def ipv4(self, name=None):
        # type: () -> DeviceList
        """Factory method to create an instance of the snappi.devicecontaineripv4.DeviceContainerIpv4 class

        Emulated ipv4 protocol
        """
        item = Device()
        item.ipv4
        self._add(item)
        return self

    def ipv6(self, name=None):
        # type: () -> DeviceList
        """Factory method to create an instance of the snappi.deviceipv6.DeviceIpv6 class

        Emulated ipv6 protocol
        """
        item = Device()
        item.ipv6
        self._add(item)
        return self

    def bgpv4(self, as_type='None', name=None):
        # type: () -> DeviceList
        """Factory method to create an instance of the snappi.devicebgpv4.DeviceBgpv4 class

        Emulated BGPv4 router and routes
        """
        item = Device()
        item.bgpv4
        self._add(item)
        return self



from .snappicommon import SnappiList
from .layer1 import Layer1


class Layer1List(SnappiList):
    def __init__(self):
        super(Layer1List, self).__init__()


    def layer1(self, port_names=None, speed='speed_10_gbps', media='None', promiscuous=False, mtu=1500, ieee_media_defaults=True, auto_negotiate=True, name=None):
        # type: () -> Layer1
        """Factory method to create an instance of the snappi.layer1.Layer1 class

        A container for layer1 settings.
        """
        item = Layer1(port_names, speed, media, promiscuous, mtu, ieee_media_defaults, auto_negotiate, name)
        self._add(item)
        return self



from .snappicommon import SnappiObject
from .deviceethernet import DeviceEthernet
from .deviceipv6 import DeviceIpv6
from .devicecontaineripv4 import DeviceContainerIpv4
from .devicebgpv4 import DeviceBgpv4


class Device(SnappiObject):
    _TYPES = {
        'ethernet': '.deviceethernet.DeviceEthernet',
        'ipv4': '.devicecontaineripv4.DeviceContainerIpv4',
        'ipv6': '.deviceipv6.DeviceIpv6',
        'bgpv4': '.devicebgpv4.DeviceBgpv4',
    }

    ETHERNET = 'ethernet'
    IPV4 = 'ipv4'
    IPV6 = 'ipv6'
    BGPV4 = 'bgpv4'

    def __init__(self, container_name=None, device_count=None, name=None):
        super(Device, self).__init__()
        self.container_name = container_name
        self.device_count = device_count
        self.name = name

    @property
    def ethernet(self):
        # type: () -> DeviceEthernet
        """Factory method to create an instance of the DeviceEthernet class

        Emulated ethernet protocol. A top level in the emulated device stack.
        """
        if 'ethernet' not in self._properties or self._properties['ethernet'] is None:
            self._properties['ethernet'] = DeviceEthernet()
        self.choice = 'ethernet'
        return self._properties['ethernet']

    @property
    def ipv4(self):
        # type: () -> DeviceContainerIpv4
        """Factory method to create an instance of the DeviceContainerIpv4 class

        Emulated ipv4 protocol
        """
        if 'ipv4' not in self._properties or self._properties['ipv4'] is None:
            self._properties['ipv4'] = DeviceContainerIpv4()
        self.choice = 'ipv4'
        return self._properties['ipv4']

    @property
    def ipv6(self):
        # type: () -> DeviceIpv6
        """Factory method to create an instance of the DeviceIpv6 class

        Emulated ipv6 protocol
        """
        if 'ipv6' not in self._properties or self._properties['ipv6'] is None:
            self._properties['ipv6'] = DeviceIpv6()
        self.choice = 'ipv6'
        return self._properties['ipv6']

    @property
    def bgpv4(self):
        # type: () -> DeviceBgpv4
        """Factory method to create an instance of the DeviceBgpv4 class

        Emulated BGPv4 router and routes
        """
        if 'bgpv4' not in self._properties or self._properties['bgpv4'] is None:
            self._properties['bgpv4'] = DeviceBgpv4()
        self.choice = 'bgpv4'
        return self._properties['bgpv4']

    @property
    def container_name(self):
        # type: () -> str
        """container_name getter

        The unique name of a Port or Lag object that will contain the emulated interfaces and/or devices.

        Returns: str
        """
        return self._properties['container_name']

    @container_name.setter
    def container_name(self, value):
        """container_name setter

        The unique name of a Port or Lag object that will contain the emulated interfaces and/or devices.

        value: str
        """
        self._properties['container_name'] = value

    @property
    def device_count(self):
        # type: () -> int
        """device_count getter

        The number of emulated protocol devices or interfaces per port.. For example if the device_count is 10 and the choice property value is ethernet then an implementation MUST create 10 ethernet interfaces. The ethernet property is a container for src, dst and eth_type properties with each on of those properties being a pattern container for 10 possible values. . If an implementation is unable to support the maximum device_count it MUST indicate what the maximum device_count is using the /results/capabilities API.. The device_count is also used by the individual child properties that are a container for a /components/schemas/Device.Pattern.

        Returns: int
        """
        return self._properties['device_count']

    @device_count.setter
    def device_count(self, value):
        """device_count setter

        The number of emulated protocol devices or interfaces per port.. For example if the device_count is 10 and the choice property value is ethernet then an implementation MUST create 10 ethernet interfaces. The ethernet property is a container for src, dst and eth_type properties with each on of those properties being a pattern container for 10 possible values. . If an implementation is unable to support the maximum device_count it MUST indicate what the maximum device_count is using the /results/capabilities API.. The device_count is also used by the individual child properties that are a container for a /components/schemas/Device.Pattern.

        value: int
        """
        self._properties['device_count'] = value

    @property
    def choice(self):
        # type: () -> Union[ethernet, ipv4, ipv6, bgpv4, choice, choice, choice, choice, choice]
        """choice getter

        The type of emulated protocol interface or device.

        Returns: Union[ethernet, ipv4, ipv6, bgpv4, choice, choice, choice, choice, choice]
        """
        return self._properties['choice']

    @choice.setter
    def choice(self, value):
        """choice setter

        The type of emulated protocol interface or device.

        value: Union[ethernet, ipv4, ipv6, bgpv4, choice, choice, choice, choice, choice]
        """
        self._properties['choice'] = value

    @property
    def name(self):
        # type: () -> str
        """name getter

        Globally unique name of an object. It also serves as the primary key for arrays of objects.

        Returns: str
        """
        return self._properties['name']

    @name.setter
    def name(self, value):
        """name setter

        Globally unique name of an object. It also serves as the primary key for arrays of objects.

        value: str
        """
        self._properties['name'] = value



from .snappicommon import SnappiObject
from .flowcounter import FlowCounter


class FlowPattern(SnappiObject):
    _TYPES = {
        'increment': '.flowcounter.FlowCounter',
        'decrement': '.flowcounter.FlowCounter',
    }

    VALUE = 'value'
    VALUES = 'values'
    INCREMENT = 'increment'
    DECREMENT = 'decrement'

    def __init__(self, metric_group=None):
        super(FlowPattern, self).__init__()
        self.metric_group = metric_group

    @property
    def increment(self):
        # type: () -> FlowCounter
        """Factory method to create an instance of the FlowCounter class

        A counter pattern that can increment or decrement.
        """
        if 'increment' not in self._properties or self._properties['increment'] is None:
            self._properties['increment'] = FlowCounter()
        self.choice = 'increment'
        return self._properties['increment']

    @property
    def decrement(self):
        # type: () -> FlowCounter
        """Factory method to create an instance of the FlowCounter class

        A counter pattern that can increment or decrement.
        """
        if 'decrement' not in self._properties or self._properties['decrement'] is None:
            self._properties['decrement'] = FlowCounter()
        self.choice = 'decrement'
        return self._properties['decrement']

    @property
    def choice(self):
        # type: () -> Union[value, values, increment, decrement, choice, choice, choice]
        """choice getter

        TBD

        Returns: Union[value, values, increment, decrement, choice, choice, choice]
        """
        return self._properties['choice']

    @choice.setter
    def choice(self, value):
        """choice setter

        TBD

        value: Union[value, values, increment, decrement, choice, choice, choice]
        """
        self._properties['choice'] = value

    @property
    def value(self):
        # type: () -> Union[string,number]
        """value getter

        TBD

        Returns: Union[string,number]
        """
        return self._properties['value']

    @value.setter
    def value(self, value):
        """value setter

        TBD

        value: Union[string,number]
        """
        self._properties['choice'] = 'value'
        self._properties['value'] = value

    @property
    def values(self):
        # type: () -> list[Union[string,number]]
        """values getter

        TBD

        Returns: list[Union[string,number]]
        """
        return self._properties['values']

    @values.setter
    def values(self, value):
        """values setter

        TBD

        value: list[Union[string,number]]
        """
        self._properties['choice'] = 'values'
        self._properties['values'] = value

    @property
    def metric_group(self):
        # type: () -> str
        """metric_group getter

        A unique name is used to indicate to the system that the field may extend the metric row key and create an aggregate metric row for every unique value. To have metric group columns appear in the flow metric rows the flow metric request allows for the metric_group value to be specified as part of the request.

        Returns: str
        """
        return self._properties['metric_group']

    @metric_group.setter
    def metric_group(self, value):
        """metric_group setter

        A unique name is used to indicate to the system that the field may extend the metric row key and create an aggregate metric row for every unique value. To have metric group columns appear in the flow metric rows the flow metric request allows for the metric_group value to be specified as part of the request.

        value: str
        """
        self._properties['metric_group'] = value



from .flowpattern import FlowPattern
from .snappicommon import SnappiObject


class FlowTcp(SnappiObject):
    _TYPES = {
        'src_port': '.flowpattern.FlowPattern',
        'dst_port': '.flowpattern.FlowPattern',
        'seq_num': '.flowpattern.FlowPattern',
        'ack_num': '.flowpattern.FlowPattern',
        'data_offset': '.flowpattern.FlowPattern',
        'ecn_ns': '.flowpattern.FlowPattern',
        'ecn_cwr': '.flowpattern.FlowPattern',
        'ecn_echo': '.flowpattern.FlowPattern',
        'ctl_urg': '.flowpattern.FlowPattern',
        'ctl_ack': '.flowpattern.FlowPattern',
        'ctl_psh': '.flowpattern.FlowPattern',
        'ctl_rst': '.flowpattern.FlowPattern',
        'ctl_syn': '.flowpattern.FlowPattern',
        'ctl_fin': '.flowpattern.FlowPattern',
        'window': '.flowpattern.FlowPattern',
    }

    def __init__(self):
        super(FlowTcp, self).__init__()

    @property
    def src_port(self):
        # type: () -> FlowPattern
        """src_port getter

        A container for packet header field patterns.A container for packet header field patterns.Tcp source port. Max length is 2 bytes.

        Returns: obj(snappi.FlowPattern)
        """
        if 'src_port' not in self._properties or self._properties['src_port'] is None:
            self._properties['src_port'] = FlowPattern()
        return self._properties['src_port']

    @property
    def dst_port(self):
        # type: () -> FlowPattern
        """dst_port getter

        A container for packet header field patterns.A container for packet header field patterns.Tcp destination port. Max length is 2 bytes.

        Returns: obj(snappi.FlowPattern)
        """
        if 'dst_port' not in self._properties or self._properties['dst_port'] is None:
            self._properties['dst_port'] = FlowPattern()
        return self._properties['dst_port']

    @property
    def seq_num(self):
        # type: () -> FlowPattern
        """seq_num getter

        A container for packet header field patterns.A container for packet header field patterns.Tcp Sequence Number. Max length is 4 bytes.

        Returns: obj(snappi.FlowPattern)
        """
        if 'seq_num' not in self._properties or self._properties['seq_num'] is None:
            self._properties['seq_num'] = FlowPattern()
        return self._properties['seq_num']

    @property
    def ack_num(self):
        # type: () -> FlowPattern
        """ack_num getter

        A container for packet header field patterns.A container for packet header field patterns.Tcp Acknowledgement Number. Max length is 4 bytes.

        Returns: obj(snappi.FlowPattern)
        """
        if 'ack_num' not in self._properties or self._properties['ack_num'] is None:
            self._properties['ack_num'] = FlowPattern()
        return self._properties['ack_num']

    @property
    def data_offset(self):
        # type: () -> FlowPattern
        """data_offset getter

        A container for packet header field patterns.A container for packet header field patterns.The number of 32 bit words in the TCP header. This indicates where the data begins. Max length is 4 bits.

        Returns: obj(snappi.FlowPattern)
        """
        if 'data_offset' not in self._properties or self._properties['data_offset'] is None:
            self._properties['data_offset'] = FlowPattern()
        return self._properties['data_offset']

    @property
    def ecn_ns(self):
        # type: () -> FlowPattern
        """ecn_ns getter

        A container for packet header field patterns.A container for packet header field patterns.Explicit congestion notification, concealment protection. Max length is 1 bit.

        Returns: obj(snappi.FlowPattern)
        """
        if 'ecn_ns' not in self._properties or self._properties['ecn_ns'] is None:
            self._properties['ecn_ns'] = FlowPattern()
        return self._properties['ecn_ns']

    @property
    def ecn_cwr(self):
        # type: () -> FlowPattern
        """ecn_cwr getter

        A container for packet header field patterns.A container for packet header field patterns.Explicit congestion notification, congestion window reduced. Max length is 1 bit.

        Returns: obj(snappi.FlowPattern)
        """
        if 'ecn_cwr' not in self._properties or self._properties['ecn_cwr'] is None:
            self._properties['ecn_cwr'] = FlowPattern()
        return self._properties['ecn_cwr']

    @property
    def ecn_echo(self):
        # type: () -> FlowPattern
        """ecn_echo getter

        A container for packet header field patterns.A container for packet header field patterns.Explicit congestion notification, echo. 1 indicates the peer is ecn capable. 0 indicates that a packet with ipv4.ecn = 11 in the ip header was received during normal transmission. Max length is 1 bit.

        Returns: obj(snappi.FlowPattern)
        """
        if 'ecn_echo' not in self._properties or self._properties['ecn_echo'] is None:
            self._properties['ecn_echo'] = FlowPattern()
        return self._properties['ecn_echo']

    @property
    def ctl_urg(self):
        # type: () -> FlowPattern
        """ctl_urg getter

        A container for packet header field patterns.A container for packet header field patterns.A value of 1 indicates that the urgent pointer field is significant. Max length is 1 bit.

        Returns: obj(snappi.FlowPattern)
        """
        if 'ctl_urg' not in self._properties or self._properties['ctl_urg'] is None:
            self._properties['ctl_urg'] = FlowPattern()
        return self._properties['ctl_urg']

    @property
    def ctl_ack(self):
        # type: () -> FlowPattern
        """ctl_ack getter

        A container for packet header field patterns.A container for packet header field patterns.A value of 1 indicates that the ackknowledgment field is significant. Max length is 1 bit.

        Returns: obj(snappi.FlowPattern)
        """
        if 'ctl_ack' not in self._properties or self._properties['ctl_ack'] is None:
            self._properties['ctl_ack'] = FlowPattern()
        return self._properties['ctl_ack']

    @property
    def ctl_psh(self):
        # type: () -> FlowPattern
        """ctl_psh getter

        A container for packet header field patterns.A container for packet header field patterns.Asks to push the buffered data to the receiving application. Max length is 1 bit.

        Returns: obj(snappi.FlowPattern)
        """
        if 'ctl_psh' not in self._properties or self._properties['ctl_psh'] is None:
            self._properties['ctl_psh'] = FlowPattern()
        return self._properties['ctl_psh']

    @property
    def ctl_rst(self):
        # type: () -> FlowPattern
        """ctl_rst getter

        A container for packet header field patterns.A container for packet header field patterns.Reset the connection. Max length is 1 bit.

        Returns: obj(snappi.FlowPattern)
        """
        if 'ctl_rst' not in self._properties or self._properties['ctl_rst'] is None:
            self._properties['ctl_rst'] = FlowPattern()
        return self._properties['ctl_rst']

    @property
    def ctl_syn(self):
        # type: () -> FlowPattern
        """ctl_syn getter

        A container for packet header field patterns.A container for packet header field patterns.Synchronize sequenece numbers. Max length is 1 bit.

        Returns: obj(snappi.FlowPattern)
        """
        if 'ctl_syn' not in self._properties or self._properties['ctl_syn'] is None:
            self._properties['ctl_syn'] = FlowPattern()
        return self._properties['ctl_syn']

    @property
    def ctl_fin(self):
        # type: () -> FlowPattern
        """ctl_fin getter

        A container for packet header field patterns.A container for packet header field patterns.Last packet from the sender. Max length is 1 bit.

        Returns: obj(snappi.FlowPattern)
        """
        if 'ctl_fin' not in self._properties or self._properties['ctl_fin'] is None:
            self._properties['ctl_fin'] = FlowPattern()
        return self._properties['ctl_fin']

    @property
    def window(self):
        # type: () -> FlowPattern
        """window getter

        A container for packet header field patterns.A container for packet header field patterns.Tcp connection window. Max length is 2 bytes.

        Returns: obj(snappi.FlowPattern)
        """
        if 'window' not in self._properties or self._properties['window'] is None:
            self._properties['window'] = FlowPattern()
        return self._properties['window']



from .flowrate import FlowRate
from .flowsize import FlowSize
from .snappicommon import SnappiObject
from .flowheaderlist import FlowHeaderList
from .flowduration import FlowDuration
from .flowtxrx import FlowTxRx


class Flow(SnappiObject):
    _TYPES = {
        'tx_rx': '.flowtxrx.FlowTxRx',
        'packet': '.flowheaderlist.FlowHeaderList',
        'size': '.flowsize.FlowSize',
        'rate': '.flowrate.FlowRate',
        'duration': '.flowduration.FlowDuration',
    }

    def __init__(self, name=None):
        super(Flow, self).__init__()
        self.name = name

    @property
    def tx_rx(self):
        # type: () -> FlowTxRx
        """tx_rx getter

        A container for different types of transmit and receive endpoint containers.The transmit and receive endpoints.

        Returns: obj(snappi.FlowTxRx)
        """
        if 'tx_rx' not in self._properties or self._properties['tx_rx'] is None:
            self._properties['tx_rx'] = FlowTxRx()
        return self._properties['tx_rx']

    @property
    def packet(self):
        # type: () -> FlowHeaderList
        """packet getter

        The header is a list of traffic protocol headers. The order of traffic protocol headers assigned to the list is the order they will appear on the wire.

        Returns: list[obj(snappi.FlowHeader)]
        """
        if 'packet' not in self._properties or self._properties['packet'] is None:
            self._properties['packet'] = FlowHeaderList()
        return self._properties['packet']

    @property
    def size(self):
        # type: () -> FlowSize
        """size getter

        The frame size which overrides the total length of the packetThe size of the packets.

        Returns: obj(snappi.FlowSize)
        """
        if 'size' not in self._properties or self._properties['size'] is None:
            self._properties['size'] = FlowSize()
        return self._properties['size']

    @property
    def rate(self):
        # type: () -> FlowRate
        """rate getter

        The rate of packet transmissionThe transmit rate of the packets.

        Returns: obj(snappi.FlowRate)
        """
        if 'rate' not in self._properties or self._properties['rate'] is None:
            self._properties['rate'] = FlowRate()
        return self._properties['rate']

    @property
    def duration(self):
        # type: () -> FlowDuration
        """duration getter

        A container for different transmit durations. The transmit duration of the packets.

        Returns: obj(snappi.FlowDuration)
        """
        if 'duration' not in self._properties or self._properties['duration'] is None:
            self._properties['duration'] = FlowDuration()
        return self._properties['duration']

    @property
    def name(self):
        # type: () -> str
        """name getter

        Globally unique name of an object. It also serves as the primary key for arrays of objects.

        Returns: str
        """
        return self._properties['name']

    @name.setter
    def name(self, value):
        """name setter

        Globally unique name of an object. It also serves as the primary key for arrays of objects.

        value: str
        """
        self._properties['name'] = value



from .snappicommon import SnappiObject


class FlowContinuous(SnappiObject):
    BYTES = 'bytes'
    NANOSECONDS = 'nanoseconds'

    def __init__(self, gap=None, delay=None, delay_unit=None):
        super(FlowContinuous, self).__init__()
        self.gap = gap
        self.delay = delay
        self.delay_unit = delay_unit

    @property
    def gap(self):
        # type: () -> int
        """gap getter

        The minimum gap between packets expressed as bytes.

        Returns: int
        """
        return self._properties['gap']

    @gap.setter
    def gap(self, value):
        """gap setter

        The minimum gap between packets expressed as bytes.

        value: int
        """
        self._properties['gap'] = value

    @property
    def delay(self):
        # type: () -> int
        """delay getter

        The delay before starting transmission of packets.

        Returns: int
        """
        return self._properties['delay']

    @delay.setter
    def delay(self, value):
        """delay setter

        The delay before starting transmission of packets.

        value: int
        """
        self._properties['delay'] = value

    @property
    def delay_unit(self):
        # type: () -> Union[bytes, nanoseconds]
        """delay_unit getter

        The delay expressed as a number of this value.

        Returns: Union[bytes, nanoseconds]
        """
        return self._properties['delay_unit']

    @delay_unit.setter
    def delay_unit(self, value):
        """delay_unit setter

        The delay expressed as a number of this value.

        value: Union[bytes, nanoseconds]
        """
        self._properties['delay_unit'] = value



from .flowpattern import FlowPattern
from .snappicommon import SnappiObject


class FlowGtpExtension(SnappiObject):
    _TYPES = {
        'extension_length': '.flowpattern.FlowPattern',
        'contents': '.flowpattern.FlowPattern',
        'next_extension_header': '.flowpattern.FlowPattern',
    }

    def __init__(self):
        super(FlowGtpExtension, self).__init__()

    @property
    def extension_length(self):
        # type: () -> FlowPattern
        """extension_length getter

        A container for packet header field patterns.An 8-bit field. This field states the length of this extension header, including the length, the contents, and the next extension header field, in 4-octet units, so the length of the extension must always be a multiple of 4.

        Returns: obj(snappi.FlowPattern)
        """
        if 'extension_length' not in self._properties or self._properties['extension_length'] is None:
            self._properties['extension_length'] = FlowPattern()
        return self._properties['extension_length']

    @property
    def contents(self):
        # type: () -> FlowPattern
        """contents getter

        A container for packet header field patterns.The extension header contents.

        Returns: obj(snappi.FlowPattern)
        """
        if 'contents' not in self._properties or self._properties['contents'] is None:
            self._properties['contents'] = FlowPattern()
        return self._properties['contents']

    @property
    def next_extension_header(self):
        # type: () -> FlowPattern
        """next_extension_header getter

        A container for packet header field patterns.An 8-bit field. It states the type of the next extension, or 0 if no next extension exists. This permits chaining several next extension headers.

        Returns: obj(snappi.FlowPattern)
        """
        if 'next_extension_header' not in self._properties or self._properties['next_extension_header'] is None:
            self._properties['next_extension_header'] = FlowPattern()
        return self._properties['next_extension_header']



from .devicepattern import DevicePattern
from .snappicommon import SnappiObject


class DeviceVlan(SnappiObject):
    _TYPES = {
        'tpid': '.devicepattern.DevicePattern',
        'priority': '.devicepattern.DevicePattern',
        'id': '.devicepattern.DevicePattern',
    }

    TPID_8100 = '8100'
    TPID_88A8 = '88a8'
    TPID_9100 = '9100'
    TPID_9200 = '9200'
    TPID_9300 = '9300'

    def __init__(self, name=None):
        super(DeviceVlan, self).__init__()
        self.name = name

    @property
    def tpid(self):
        # type: () -> DevicePattern
        """tpid getter

        A container for emulated device property patterns.Vlan tag protocol identifier.

        Returns: obj(snappi.DevicePattern)
        """
        if 'tpid' not in self._properties or self._properties['tpid'] is None:
            self._properties['tpid'] = DevicePattern()
        return self._properties['tpid']

    @property
    def priority(self):
        # type: () -> DevicePattern
        """priority getter

        A container for emulated device property patterns.Vlan priority.

        Returns: obj(snappi.DevicePattern)
        """
        if 'priority' not in self._properties or self._properties['priority'] is None:
            self._properties['priority'] = DevicePattern()
        return self._properties['priority']

    @property
    def id(self):
        # type: () -> DevicePattern
        """id getter

        A container for emulated device property patterns.Vlan id.

        Returns: obj(snappi.DevicePattern)
        """
        if 'id' not in self._properties or self._properties['id'] is None:
            self._properties['id'] = DevicePattern()
        return self._properties['id']

    @property
    def name(self):
        # type: () -> str
        """name getter

        Globally unique name of an object. It also serves as the primary key for arrays of objects.

        Returns: str
        """
        return self._properties['name']

    @name.setter
    def name(self, value):
        """name setter

        Globally unique name of an object. It also serves as the primary key for arrays of objects.

        value: str
        """
        self._properties['name'] = value



from .snappicommon import SnappiObject


class Bgpv4Metric(SnappiObject):
    def __init__(self, name=None, sessions_total=None, sessions_up=None, sessions_down=None, sessions_not_started=None, routes_advertised=None, routes_withdrawn=None):
        super(Bgpv4Metric, self).__init__()
        self.name = name
        self.sessions_total = sessions_total
        self.sessions_up = sessions_up
        self.sessions_down = sessions_down
        self.sessions_not_started = sessions_not_started
        self.routes_advertised = routes_advertised
        self.routes_withdrawn = routes_withdrawn

    @property
    def name(self):
        # type: () -> str
        """name getter

        The name of a configured BGPv4 Object.

        Returns: str
        """
        return self._properties['name']

    @name.setter
    def name(self, value):
        """name setter

        The name of a configured BGPv4 Object.

        value: str
        """
        self._properties['name'] = value

    @property
    def sessions_total(self):
        # type: () -> int
        """sessions_total getter

        Total number of session

        Returns: int
        """
        return self._properties['sessions_total']

    @sessions_total.setter
    def sessions_total(self, value):
        """sessions_total setter

        Total number of session

        value: int
        """
        self._properties['sessions_total'] = value

    @property
    def sessions_up(self):
        # type: () -> int
        """sessions_up getter

        Sessions are in active state

        Returns: int
        """
        return self._properties['sessions_up']

    @sessions_up.setter
    def sessions_up(self, value):
        """sessions_up setter

        Sessions are in active state

        value: int
        """
        self._properties['sessions_up'] = value

    @property
    def sessions_down(self):
        # type: () -> int
        """sessions_down getter

        Sessions are not active state

        Returns: int
        """
        return self._properties['sessions_down']

    @sessions_down.setter
    def sessions_down(self, value):
        """sessions_down setter

        Sessions are not active state

        value: int
        """
        self._properties['sessions_down'] = value

    @property
    def sessions_not_started(self):
        # type: () -> int
        """sessions_not_started getter

        Sessions not able to start due to some internal issue

        Returns: int
        """
        return self._properties['sessions_not_started']

    @sessions_not_started.setter
    def sessions_not_started(self, value):
        """sessions_not_started setter

        Sessions not able to start due to some internal issue

        value: int
        """
        self._properties['sessions_not_started'] = value

    @property
    def routes_advertised(self):
        # type: () -> int
        """routes_advertised getter

        Number of advertised routes sent

        Returns: int
        """
        return self._properties['routes_advertised']

    @routes_advertised.setter
    def routes_advertised(self, value):
        """routes_advertised setter

        Number of advertised routes sent

        value: int
        """
        self._properties['routes_advertised'] = value

    @property
    def routes_withdrawn(self):
        # type: () -> int
        """routes_withdrawn getter

        Number of routes withdrawn

        Returns: int
        """
        return self._properties['routes_withdrawn']

    @routes_withdrawn.setter
    def routes_withdrawn(self, value):
        """routes_withdrawn setter

        Number of routes withdrawn

        value: int
        """
        self._properties['routes_withdrawn'] = value



from .flowsizeincrement import FlowSizeIncrement
from .flowsizerandom import FlowSizeRandom
from .snappicommon import SnappiObject


class FlowSize(SnappiObject):
    _TYPES = {
        'increment': '.flowsizeincrement.FlowSizeIncrement',
        'random': '.flowsizerandom.FlowSizeRandom',
    }

    FIXED = 'fixed'
    INCREMENT = 'increment'
    RANDOM = 'random'

    def __init__(self):
        super(FlowSize, self).__init__()

    @property
    def increment(self):
        # type: () -> FlowSizeIncrement
        """Factory method to create an instance of the FlowSizeIncrement class

        Frame size that increments from a starting size to an ending size incrementing by a step size.
        """
        if 'increment' not in self._properties or self._properties['increment'] is None:
            self._properties['increment'] = FlowSizeIncrement()
        self.choice = 'increment'
        return self._properties['increment']

    @property
    def random(self):
        # type: () -> FlowSizeRandom
        """Factory method to create an instance of the FlowSizeRandom class

        Random frame size from a min value to a max value.
        """
        if 'random' not in self._properties or self._properties['random'] is None:
            self._properties['random'] = FlowSizeRandom()
        self.choice = 'random'
        return self._properties['random']

    @property
    def choice(self):
        # type: () -> Union[fixed, increment, random, choice, choice, choice]
        """choice getter

        TBD

        Returns: Union[fixed, increment, random, choice, choice, choice]
        """
        return self._properties['choice']

    @choice.setter
    def choice(self, value):
        """choice setter

        TBD

        value: Union[fixed, increment, random, choice, choice, choice]
        """
        self._properties['choice'] = value

    @property
    def fixed(self):
        # type: () -> int
        """fixed getter

        TBD

        Returns: int
        """
        return self._properties['fixed']

    @fixed.setter
    def fixed(self, value):
        """fixed setter

        TBD

        value: int
        """
        self._properties['choice'] = 'fixed'
        self._properties['fixed'] = value
